=== modified file '.bzrignore'
--- .bzrignore	2005-06-09 00:49:58 +0000
+++ .bzrignore	2005-09-22 22:45:21 +0000
@@ -6,3 +6,4 @@
 debian/bzrtools.substvars
 debian/files
 debian/stamp-patched
+,,*

=== modified file 'dotgraph.py'
--- dotgraph.py	2005-09-22 21:31:04 +0000
+++ dotgraph.py	2005-09-22 23:08:00 +0000
@@ -6,6 +6,7 @@
 import errno
 import tempfile
 import shutil
+import time
 
 RSVG_OUTPUT_TYPES = ('png', 'jpg')
 DOT_OUTPUT_TYPES = ('svg', 'svgz', 'gif', 'jpg', 'ps', 'fig', 'mif', 'png', 
@@ -21,7 +22,7 @@
 
 class Node(object):
     def __init__(self, name, color=None, label=None, rev_id=None,
-                 cluster=None, node_style=None):
+                 cluster=None, node_style=None, date=None, message=None):
         self.name = name
         self.color = color
         self.label = label
@@ -31,6 +32,8 @@
             self.node_style = []
         self.cluster = cluster
         self.rank = None
+        self.date = date
+        self.message = message
         self.href = None
 
     def define(self):
@@ -46,8 +49,18 @@
         if label is not None:
             attributes.append('label="%s"' % label)
         attributes.append('shape="box"')
+        tooltip = ''
+        if self.message is not None:
+            tooltip += self.message.replace('"', '\\"')
+        # This makes the tooltip too long
+        # if self.rev_id is not None:
+        #     tooltip += ' revid:%s' % self.rev_id
+        if tooltip:
+            attributes.append('tooltip="%s"' % tooltip)
         if self.href is not None:
             attributes.append('href="%s"' % self.href)
+        elif tooltip:
+            attributes.append('href="#"')
         if len(attributes) > 0:
             return '%s[%s]' % (self.name, " ".join(attributes))
 
@@ -190,3 +203,34 @@
         dot_proc.stdin.write(line)
     dot_proc.stdin.close()
     return dot_proc.wait()
+
+def invoke_dot_html(input, out_file):
+    """\
+    Produce an html file, which uses a .png file, and a cmap to provide
+    annotated revisions.
+    """
+    tempdir = tempfile.mkdtemp()
+    try:
+        temp_dot = os.path.join(tempdir, 'temp.dot')
+        status = invoke_dot(input, temp_dot, file_type='dot')
+
+        dot = open(temp_dot)
+        temp_file = os.path.join(tempdir, 'temp.cmapx')
+        status = invoke_dot(dot, temp_file, 'cmapx')
+
+        png_file = '.'.join(out_file.split('.')[:-1] + ['png'])
+        dot.seek(0)
+        status = invoke_dot(dot, png_file, 'png')
+
+        png_relative = png_file.split('/')[-1]
+        html = open(out_file, 'wb')
+        w = html.write
+        w('<html><head><title></title></head>\n')
+        w('<body>\n')
+        w('<img src="%s" usemap="#G" border=0/>' % png_relative)
+        w(open(temp_file).read())
+        w('</body></html>\n')
+    finally:
+        shutil.rmtree(tempdir)
+    return status
+

=== modified file 'graph.py'
--- graph.py	2005-09-22 23:24:38 +0000
+++ graph.py	2005-09-23 00:02:14 +0000
@@ -1,4 +1,4 @@
-from dotgraph import Node, dot_output, invoke_dot, invoke_dot_aa, NoDot, NoRsvg
+from dotgraph import Node, dot_output, invoke_dot, invoke_dot_aa, invoke_dot_html, NoDot, NoRsvg
 from dotgraph import RSVG_OUTPUT_TYPES, DOT_OUTPUT_TYPES, Edge
 from bzrlib.branch import Branch
 from bzrlib.errors import BzrCommandError, NoCommonRoot, NoSuchRevision
@@ -10,6 +10,7 @@
 import bzrlib.errors
 import re
 import os.path
+import time
 
 mail_map = {'aaron.bentley@utoronto.ca'     : 'Aaron Bentley',
             'abentley@panoramicfeedback.com': 'Aaron Bentley',
@@ -61,16 +62,27 @@
             new_ancestors[me][new_parent] = distance
     return new_ancestors    
 
-def get_committer(rev_id, source):
+def get_rev_info(rev_id, source):
+    """Return the committer, message, and date of a revision."""
+    committer = None
+    message = None
+    date = None
+    if rev_id == 'null:':
+        return None, 'Null Revision', None
     try:
-        committer = short_committer(source.get_revision(rev_id).committer)
+        rev = source.get_revision(rev_id)
     except NoSuchRevision:
         try:
             committer = '-'.join(rev_id.split('-')[:-2]).strip(' ')
             if committer == '':
-                return None
+                return None, None, None
         except ValueError:
-            return None
+            return None, None, None
+    else:
+        committer = short_committer(rev.committer)
+        if rev.message is not None:
+            message = rev.message.split('\n')[0]
+        date = time.strftime('%Y-%m-%d', time.gmtime(rev.timestamp + (rev.timezone or 0)))
     if '@' in committer:
         try:
             committer = mail_map[committer]
@@ -80,7 +92,7 @@
         committer = committer_alias[committer]
     except KeyError:
         pass
-    return committer
+    return committer, message, date
 
 class Grapher(object):
     def __init__(self, branch, other_branch=None):
@@ -111,12 +123,6 @@
             self.base = None
             self.m_history = []
 
-    def get_timestamp(self, revision_id):
-        try:
-            return float(self.branch.get_revision(revision_id).timestamp)
-        except NoSuchRevision:
-            return None
-
     def dot_node(self, node, num):
         try:
             n_rev = self.n_history.index(node) + 1
@@ -157,14 +163,12 @@
             color = "#33ff99"
 
         label = [name]
-        committer = get_committer(node, self.branch)
+        committer, message, date = get_rev_info(node, self.branch)
         if committer is not None:
             label.append(committer)
 
-        timestamp = self.get_timestamp(node)
-        if timestamp is not None:
-            date = datetime.fromtimestamp(timestamp)
-            label.append("%d/%d/%d" % (date.year, date.month, date.day))
+        if date is not None:
+            label.append(date)
 
         if node in self.distances:
             rank = self.distances[node]
@@ -173,7 +177,8 @@
             rank = None
 
         d_node = Node("n%d" % num, color=color, label="\\n".join(label), 
-                    rev_id=node, cluster=cluster)
+                    rev_id=node, cluster=cluster, message=message,
+                    date=date)
         d_node.rank = rank
 
         if node not in self.ancestors:
@@ -248,6 +253,12 @@
         my_file = file(filename, 'wb')
         for fragment in output:
             my_file.write(fragment)
+    elif ext=='html':
+        try:
+            invoke_dot_html(output, filename)
+        except NoDot, e:
+            raise BzrCommandError("Can't find 'dot'.  Please ensure Graphviz"\
+                " is installed correctly, or use --noantialias")
     elif not done:
         print "Unknown file extension: %s" % ext
 

